@@ -7,6 +7,7 @@
 #include "efaxmlparser.h"
 #include "logging.h"
 #include "scopedxmlstreamreader.h"
+#include "ifopt/ifoptutil.h"
 
 #include <KPublicTransport/Journey>
 #include <KPublicTransport/Location>
@@ -18,32 +19,60 @@
 
 using namespace KPublicTransport;
 
-Location EfaXmlParser::parseItdOdvAssignedStop(const ScopedXmlStreamReader &reader) const
+void EfaXmlParser::parseLocationCommon(Location &loc, const ScopedXmlStreamReader &reader) const
 {
-    Location loc;
     if (reader.attributes().hasAttribute(QLatin1String("x")) && reader.attributes().hasAttribute(QLatin1String("y"))) {
         loc.setLatitude(reader.attributes().value(QLatin1String("y")).toDouble());
         loc.setLongitude(reader.attributes().value(QLatin1String("x")).toDouble());
     }
-    loc.setIdentifier(m_locationIdentifierType, reader.attributes().value(QLatin1String("stopID")).toString());
+
+    // can be already set on loc, so don't reset it if missing here
+    const auto locality = reader.attributes().value(QLatin1String("place")).toString();
+    if (!locality.isEmpty()) {
+        loc.setLocality(locality);
+    }
+
+    const auto id = reader.attributes().value(QLatin1String("stopID")).toString();
+    if (!id.isEmpty()) {
+        if (!isDummyStopId(id)) {
+            loc.setIdentifier(m_locationIdentifierType, id);
+        }
+    } else {
+        loc.setIdentifier(m_locationIdentifierType, reader.attributes().value(QLatin1String("stateless")).toString());
+    }
+
+    // check if IFOPT identifiers are present, in decreasing level of detail
+    for (const auto &attr : {QLatin1String("pointGid"), QLatin1String("areaGid"), QLatin1String("gid")}) {
+        const auto id = reader.attributes().value(attr);
+        if (IfoptUtil::isValid(id)) {
+            loc.setIdentifier(QStringLiteral("ifopt"), id.toString());
+            break;
+        }
+    }
+
+    const auto type = reader.attributes().value(QLatin1String("anyType"));
+    if (type == QLatin1String("stop")) {
+        loc.setType(Location::Stop);
+    } else if (!type.isEmpty()) {
+        qCDebug(Log) << "Unhandled EFA location type:" << type;
+    } else if (!id.isEmpty() && !isDummyStopId(id)) {
+        loc.setType(Location::Stop);
+    }
+}
+
+Location EfaXmlParser::parseItdOdvAssignedStop(const ScopedXmlStreamReader &reader) const
+{
+    Location loc;
+    parseLocationCommon(loc, reader);
     loc.setName(reader.attributes().value(QLatin1String("nameWithPlace")).toString());
-    loc.setLocality(reader.attributes().value(QLatin1String("place")).toString());
+    loc.setType(Location::Stop);
     return loc;
 }
 
 Location EfaXmlParser::parseOdvNameElem(ScopedXmlStreamReader &reader) const
 {
     Location loc;
-    if (reader.attributes().hasAttribute(QLatin1String("x")) && reader.attributes().hasAttribute(QLatin1String("y"))) {
-        loc.setLatitude(reader.attributes().value(QLatin1String("y")).toDouble());
-        loc.setLongitude(reader.attributes().value(QLatin1String("x")).toDouble());
-    }
-    const auto id = reader.attributes().value(QLatin1String("stopID")).toString();
-    if (!id.isEmpty()) {
-        loc.setIdentifier(m_locationIdentifierType, id);
-    } else {
-        loc.setIdentifier(m_locationIdentifierType, reader.attributes().value(QLatin1String("stateless")).toString());
-    }
+    parseLocationCommon(loc, reader);
     loc.setName(reader.readElementText(QXmlStreamReader::SkipChildElements));
     return loc;
 }
@@ -90,6 +119,15 @@ static QDateTime parseDateTime(ScopedXmlStreamReader &&reader)
     return dt;
 }
 
+struct {
+    const char *name;
+    Load::Category category;
+} static constexpr const occupancy_load_map[] = {
+    { "MANY_SEATS", Load::Low },
+    { "FEW_SEATS", Load::Medium },
+    { "STANDING_ONLY", Load::High },
+};
+
 Stopover EfaXmlParser::parseDmDeparture(ScopedXmlStreamReader &&reader) const
 {
     Stopover dep;
@@ -102,14 +140,23 @@ Stopover EfaXmlParser::parseDmDeparture(ScopedXmlStreamReader &&reader) const
         stop = stopIt.value();
     }
 
+    parseLocationCommon(stop, reader);
     stop.setName(reader.attributes().value(QLatin1String("stopName")).toString());
-    if (reader.attributes().hasAttribute(QLatin1String("x")) && reader.attributes().hasAttribute(QLatin1String("y"))) {
-        stop.setLatitude(reader.attributes().value(QLatin1String("y")).toFloat());
-        stop.setLongitude(reader.attributes().value(QLatin1String("x")).toFloat());
-    }
-    stop.setIdentifier(m_locationIdentifierType, stopId);
+    stop.setType(Location::Stop);
     dep.setStopPoint(stop);
 
+    const auto occupancy = reader.attributes().value(QLatin1String("occupancy"));
+    if (!occupancy.isEmpty()) {
+        LoadInfo load;
+        for (const auto &map : occupancy_load_map) {
+            if (occupancy == QLatin1String(map.name)) {
+                load.setLoad(map.category);
+                break;
+            }
+        }
+        dep.setLoadInformation({load});
+    }
+
     while (reader.readNextSibling()) {
         if (reader.name() == QLatin1String("itdServingLine")) {
             Line line;
@@ -149,11 +196,8 @@ std::vector<Stopover> EfaXmlParser::parseDmResponse(const QByteArray &data) cons
 void EfaXmlParser::parseTripDeparture(ScopedXmlStreamReader &&reader, JourneySection &section) const
 {
     Location loc;
+    parseLocationCommon(loc, reader);
     loc.setName(reader.attributes().value(QLatin1String("name")).toString());
-    loc.setLatitude(reader.attributes().value(QLatin1String("y")).toFloat());
-    loc.setLongitude(reader.attributes().value(QLatin1String("x")).toFloat());
-    loc.setIdentifier(m_locationIdentifierType, reader.attributes().value(QLatin1String("stopID")).toString());
-    loc.setLocality(reader.attributes().value(QLatin1String("place")).toString());
 
     section.setFrom(loc);
     // ### are those the correct ones? there's also just "platform"
@@ -172,11 +216,8 @@ void EfaXmlParser::parseTripDeparture(ScopedXmlStreamReader &&reader, JourneySec
 void EfaXmlParser::parseTripArrival(ScopedXmlStreamReader &&reader, JourneySection &section) const
 {
     Location loc;
+    parseLocationCommon(loc, reader);
     loc.setName(reader.attributes().value(QLatin1String("name")).toString());
-    loc.setLatitude(reader.attributes().value(QLatin1String("y")).toFloat());
-    loc.setLongitude(reader.attributes().value(QLatin1String("x")).toFloat());
-    loc.setIdentifier(m_locationIdentifierType, reader.attributes().value(QLatin1String("stopID")).toString());
-    loc.setLocality(reader.attributes().value(QLatin1String("place")).toString());
 
     section.setTo(loc);
     // ### are those the correct ones? there's also just "platform"
@@ -195,11 +236,8 @@ void EfaXmlParser::parseTripArrival(ScopedXmlStreamReader &&reader, JourneySecti
 Stopover EfaXmlParser::parsePartialTripIntermediateStop(ScopedXmlStreamReader &&reader) const
 {
     Location loc;
+    parseLocationCommon(loc, reader);
     loc.setName(reader.attributes().value(QLatin1String("name")).toString());
-    loc.setLatitude(reader.attributes().value(QLatin1String("y")).toFloat());
-    loc.setLongitude(reader.attributes().value(QLatin1String("x")).toFloat());
-    loc.setIdentifier(m_locationIdentifierType, reader.attributes().value(QLatin1String("stopID")).toString());
-    loc.setLocality(reader.attributes().value(QLatin1String("place")).toString());
 
     Stopover stop;
     stop.setStopPoint(loc);
@@ -298,6 +336,13 @@ JourneySection EfaXmlParser::parseTripPartialRoute(ScopedXmlStreamReader &&reade
             }
         } else if (reader.name() == QLatin1String("infoLink")) {
             section.addNotes(parseInfoLink(reader.subReader()));
+        } else if (reader.name() == QLatin1String("itdInfoTextList")) {
+            auto subReader = reader.subReader();
+            while (subReader.readNextSibling()) {
+                if (subReader.name() == QLatin1String("infoTextListElem")) {
+                    section.addNote(subReader.readElementText());
+                }
+            }
         } else if (reader.name() == QLatin1String("itdStopSeq")) {
             section.setIntermediateStops(parsePartialTripStopSequence(reader.subReader()));
         } else if (reader.name() == QLatin1String("itdPathCoordinates")) {
