@@ -400,8 +400,11 @@ std::vector<Location> HafasMgateParser::parseLocations(const QByteArray &data) c
     return {};
 }
 
-std::vector<Journey> HafasMgateParser::parseJourneys(const QByteArray &data) const
+std::vector<Journey> HafasMgateParser::parseJourneys(const QByteArray &data)
 {
+    m_nextJourneyContext.clear();
+    m_previousJourneyContext.clear();
+
     const auto topObj = QJsonDocument::fromJson(data).object();
     if (!parseError(topObj)) {
         return {};
@@ -481,7 +484,7 @@ static std::vector<Path> parsePaths(const QJsonArray &polyL, const std::vector<L
         sections.reserve(ppLocRefL.size() - 1);
         int prevPpIdx = 0;
         QPointF prevCoord;
-        for (const auto ppLocRefV : ppLocRefL) {
+        for (const auto &ppLocRefV : ppLocRefL) {
             const auto ppLocRef = ppLocRefV.toObject();
             const auto ppIdx = ppLocRef.value(QLatin1String("ppIdx")).toInt();
             if (ppIdx == 0 || ppIdx < prevPpIdx) {
@@ -531,7 +534,7 @@ static Path parsePolyG(const QJsonObject &obj, const std::vector<Path> &paths)
     return {};
 }
 
-std::vector<Journey> HafasMgateParser::parseTripSearch(const QJsonObject &obj) const
+std::vector<Journey> HafasMgateParser::parseTripSearch(const QJsonObject &obj)
 {
     const auto commonObj = obj.value(QLatin1String("common")).toObject();
     const auto icos = parseIcos(commonObj.value(QLatin1String("icoL")).toArray());
@@ -683,6 +686,9 @@ std::vector<Journey> HafasMgateParser::parseTripSearch(const QJsonObject &obj) c
         res.push_back(journey);
     }
 
+    m_previousJourneyContext = obj.value(QLatin1String("outCtxScrB")).toString();
+    m_nextJourneyContext =  obj.value(QLatin1String("outCtxScrF")).toString();
+
     return res;
 }
 
