@@ -7,6 +7,7 @@
 #include "opentripplannerparser.h"
 #include "gtfs/hvt.h"
 #include "geo/polylinedecoder_p.h"
+#include "ifopt/ifoptutil.h"
 
 #include <KPublicTransport/Journey>
 #include <KPublicTransport/RentalVehicle>
@@ -22,8 +23,9 @@
 
 using namespace KPublicTransport;
 
-OpenTripPlannerParser::OpenTripPlannerParser(const QString &identifierType)
+OpenTripPlannerParser::OpenTripPlannerParser(const QString &identifierType, const QString &ifoptPrefix)
     : m_identifierType(identifierType)
+    , m_ifoptPrefix(ifoptPrefix)
 {
 }
 
@@ -87,6 +89,12 @@ bool OpenTripPlannerParser::parseLocationFragment(const QJsonObject &obj, Locati
     if (!id.isEmpty()) {
         loc.setIdentifier(m_identifierType, id);
     }
+    if (!m_ifoptPrefix.isEmpty() && id.size() > m_ifoptPrefix.size() + 1 && id.startsWith(m_ifoptPrefix) && id.at(m_ifoptPrefix.size()) == QLatin1Char(':')) {
+        const auto ifopt = QStringView(id).mid(m_ifoptPrefix.size() + 1);
+        if (IfoptUtil::isValid(ifopt)) {
+            loc.setIdentifier(QStringLiteral("ifopt"), ifopt.toString());
+        }
+    }
 
     const auto bss = obj.value(QLatin1String("bikeRentalStation")).toObject();
     if (!bss.isEmpty()) {
@@ -95,6 +103,13 @@ bool OpenTripPlannerParser::parseLocationFragment(const QJsonObject &obj, Locati
         return loc.rentalVehicleStation().network().isValid() || loc.rentalVehicle().network().isValid();
     }
 
+    const auto mode = obj.value(QLatin1String("vehicleMode")).toString();
+    if (mode == QLatin1String("CARPOOL")) {
+        loc.setType(Location::CarpoolPickupDropoff);
+    } else if (!mode.isEmpty() && loc.type() == Location::Place) {
+        loc.setType(Location::Stop);
+    }
+
     return true;
 }
 
@@ -471,15 +486,20 @@ Journey OpenTripPlannerParser::parseJourney(const QJsonObject &obj) const
     return journey;
 }
 
-std::vector<Journey> OpenTripPlannerParser::parseJourneys(const QJsonObject& obj) const
+std::vector<Journey> OpenTripPlannerParser::parseJourneys(const QJsonObject& obj)
 {
     std::vector<Journey> journeys;
 
-    const auto journeysArray = obj.value(QLatin1String("plan")).toObject().value(QLatin1String("itineraries")).toArray();
+    const auto plan = obj.value(QLatin1String("plan")).toObject();
+    const auto journeysArray = plan.value(QLatin1String("itineraries")).toArray();
     journeys.reserve(journeysArray.size());
     for (const auto &journeyObj : journeysArray) {
         journeys.push_back(parseJourney(journeyObj.toObject()));
     }
 
+    m_nextJourneyContext.dateTime = parseJourneyDateTime(plan.value(QLatin1String("nextDateTime")));
+    m_prevJourneyContext.dateTime = parseJourneyDateTime(plan.value(QLatin1String("prevDateTime")));
+    m_nextJourneyContext.searchWindow = m_prevJourneyContext.searchWindow = plan.value(QLatin1String("searchWindowUsed")).toInt();
+
     return journeys;
 }
