@@ -10,6 +10,7 @@
 #include <QColor>
 #include <QDateTime>
 #include <QDebug>
+#include <QLocale>
 #include <QMetaObject>
 #include <QMetaProperty>
 #include <QRectF>
@@ -21,6 +22,32 @@
 
 using namespace KPublicTransport;
 
+QString Json::translatedValue(const QJsonObject &obj, const QString &key)
+{
+    auto languageWithCountry = QLocale().name();
+    auto it = obj.constFind(key + QLatin1Char('[') + languageWithCountry + QLatin1Char(']'));
+    if (it != obj.constEnd()) {
+        return it.value().toString();
+    }
+    const auto language = languageWithCountry.midRef(0, languageWithCountry.indexOf(QLatin1Char('_')));
+    it = obj.constFind(key + QLatin1Char('[') + language + QLatin1Char(']'));
+    if (it != obj.constEnd()) {
+        return it.value().toString();
+    }
+    return obj.value(key).toString();
+}
+
+QStringList Json::toStringList(const QJsonValue &v)
+{
+    const auto a = v.toArray();
+    QStringList l;
+    l.reserve(a.size());
+    for (const auto &av : a) {
+        l.push_back(av.toString());
+    }
+    return l;
+}
+
 static QJsonValue variantToJson(const QVariant &v)
 {
     switch (v.userType()) {
@@ -148,15 +175,7 @@ static QVariant variantFromJson(const QJsonValue &v, int mt)
         case QVariant::Url:
             return QUrl(v.toString());
         case QVariant::StringList:
-        {
-            const auto a = v.toArray();
-            QStringList l;
-            l.reserve(a.size());
-            for (const auto &av : a) {
-                l.push_back(av.toString());
-            }
-            return l;
-        }
+            return Json::toStringList(v);
         case QVariant::RectF:
         {
             const auto obj = v.toObject();
